!---------------------------------------------------------------------
!     Copyright (C) GFD Dennou Club, 2005. All rights reserved.
!---------------------------------------------------------------------
! physics_ground.f90 
! SST, ڤɽ̤̽ɤ߹
!
! History
!   2005/09/20 Yamada Yukiko     create
!

module physics_ground_mod

  use type_mod,    only : REKIND, DBKIND, INTKIND, TOKEN, STRING

  implicit none

  private
  public :: physics_ground

contains

  subroutine physics_ground( &
       &  xy_SurfTemp   ,& ! (out) ɽ
       &  y_Lat         ) ! (in) ٺɸ

!!!! AGCM5 ¾Ǥ餷ɽ̾ !!!!!
!       & xy_SurfTemp                  , & ! ɽ     = SST
!       & xy_SurfHeatCapa              , & ! ɽǮ   = 0.0
!       & xy_SurfAlbedo                , & ! ɽ٥ = 0.15
!       & xy_SurfHumidCoeff            , & ! ɽ   = 1.0
!       & xy_SurfRoughLength           , & ! ɽĹ   = 0.0001
!      REAL       GRZSD  ( IDIM*JDIM )    !" ɽʬ = 0.0
!      INTEGER    IDSFC  ( IDIM*JDIM )    !" ɽ     = 0.0
!      REAL       GFLUX  ( IDIM*JDIM )    !" Ǯեå = 0.0


    !==== Dependency
    use type_mod,      only: REKIND, DBKIND, INTKIND, TOKEN, STRING
    use grid_3d_mod,   only: im, jm, km
    use dc_trace,      only: SetDebug, BeginSub, EndSub, DbgMessage, DataDump

    implicit none

    !==== Input
    !
    real(DBKIND), intent(in) ::   &
         &  y_Lat(:)                ! (in) ٺɸ

    !==== Output
    !
    real(DBKIND), intent(out) ::  &
         & xy_SurfTemp(im,jm)       ! (out) ɽ̲

    !----- ѿ -----
    character(STRING),  parameter:: subname = "physics_ground"
    real(DBKIND)        ::  & 
         & xy_SeaSurfaceTemp(im,jm)            ! ɽ̲

    ! Hosaka et al. (1998) SST ѥѥ᡼
    real(DBKIND), parameter :: TEQ   = 302.0d0
    real(DBKIND), parameter :: ALAT0 =   0.0d0
    real(DBKIND), parameter :: ALAT1 =  30.0d0
    real(DBKIND), parameter :: ALPHA =  60.0d0
    real(DBKIND), parameter :: BETA  =  32.0d0
    real(DBKIND), parameter :: GAMMA =   0.0d0
    real(DBKIND), parameter :: ALACON = 7.0d0

    continue

    !----------------------------------------------------------------
    !   Ͻ
    !----------------------------------------------------------------
    call BeginSub(subname)

    !----------------------------------------------------------------
    !   ɽ̲٤
    !----------------------------------------------------------------
    
    !----- ̲٤ -----
    ! ϥե뤫ɤ߽Ф٤Ū. 
    ! Hosaka et al. (1998)  SST ʬۤͿ. 
    
    call mksst( &
         & xy_SeaSurfaceTemp                           , &
         & y_Lat                                       , &
         & TEQ , ALAT0 , ALAT1 , ALPHA , BETA  , GAMMA , &
         & ALACON                                       )


    !----- ɽ̲٤̲٤֤-----
    xy_SurfTemp = xy_SeaSurfaceTemp


    !----------------------------------------------------------------
    !   λ
    !----------------------------------------------------------------
    call EndSub(subname)

  end subroutine physics_ground


  subroutine mksst( &
    & xy_SeaSurfaceTemp                           , &
    & y_Lat                                       , &
    & TEQ , ALAT0 , ALAT1 , ALPHA , BETA  , GAMMA , &
    & ALACON                                       )

    !==== Dependency
    use type_mod,      only: REKIND, DBKIND, INTKIND, TOKEN, STRING
    use grid_3d_mod,   only: im, jm, km
    use constants_mod, only: PI 
    use dc_trace,      only: SetDebug, BeginSub, EndSub, DbgMessage, DataDump

    implicit none

    !==== Input
    !
    real(DBKIND), intent(in) ::   &
         &  y_Lat(:)             ,& ! (in) ٺɸ
         &  TEQ , ALAT0 , ALAT1 , ALPHA , BETA  , GAMMA , ALACON

    !==== Output
    !
    real(DBKIND), intent(out) ::  &
         & xy_SeaSurfaceTemp(im,jm)       ! (out) ɽ̲

    !----- ѿ -----
    character(STRING),  parameter:: subname = "mksst"

    ! do 롼Ѻѿ ( i* j*ľ k*ȿ l*)
    integer(INTKIND)    :: i, j

    real(DBKIND) :: PHI1, AB4, PHI, GSSTP, ALATP, ALATM, GSSTMX
    integer(INTKIND)    :: JP, JMM

    continue

    !----------------------------------------------------------------
    !   Ͻ
    !----------------------------------------------------------------
    call BeginSub(subname)

    !----------------------------------------------------------------
    !   ɽ̲٤
    !----------------------------------------------------------------

    PHI1 = ABS( ALAT1 * PI/180. )
    AB4  = 2. *( PHI1**3 )*BETA/ALPHA
    
    do j = 1, jm
       PHI   = ABS( y_Lat( J )*PI/180. - ALAT0 *PI/180. )
       GSSTP = TEQ                                                      &
            &         - ALPHA/2.                                        &
            &           * ( PHI - MAX(   SQRT(  PHI1**2+AB4 )      &
            &                     - SQRT( (PHI-PHI1)**2+AB4 ), 0.  )  ) &
            &         + GAMMA *( PHI**3 )
       do i = 1, im
          xy_SeaSurfaceTemp(i,j) = GSSTP
       end do
    end do
        
    ! ---- 濴 ALAT0 +/- ALACON δ֤ʿó -----
    ALATP = ( ALAT0 + ALACON )*PI/180.
    ALATM = ( ALAT0 - ALACON )*PI/180.

    JP = 1
    JMM = jm
    do  j = 1, jm
       IF ( (-y_Lat( J )*PI/180.) .GT. ALATP ) THEN
          JMM = J
       ENDIF
       IF ( (-y_Lat( J )*PI/180.) .GE. ALATM ) THEN
          JP = J
       ENDIF
    end do

    GSSTMX = ( xy_SeaSurfaceTemp(1,JP) * ( ALATP - y_Lat(JP+1)*PI/180.)     &
         &   +  xy_SeaSurfaceTemp(1,JP+1) * ( y_Lat(JP)*PI/180. - ALATP ))  &
         &        /( y_Lat(JP)*PI/180. - y_Lat(JP+1)*PI/180. ) 

    do j = JMM+1, JP 
       do i = 1, im 
          xy_SeaSurfaceTemp(i,j) = GSSTMX
       end do
    end do

    !----------------------------------------------------------------
    !   λ
    !----------------------------------------------------------------
    call EndSub(subname)

  end subroutine mksst

end module physics_ground_mod




